#!/usr/bin/env python3

import subprocess
import os
import shutil
import psutil
import dbus
import time
import sys

Failed_String = '\033[031mfailed\033[0m'
# Succeed_String='\033[032msucceed\033[0m'
Succeed_String = 'succeed'

kydbus_police_json="/etc/kydbus-police/.kydbus-police.json"
kydbus_police_key="麒麟软件-警务终端项目-Kydbus_Police-Initialize-State"

with open(kydbus_police_json, 'w') as file:
    pass

interface_authority_switch=False


# ---------------------------------------------------------------------


# 创建一个空的类，用于模拟不输出任何内容的文件对象
class NullWriter:
    def write(self, *args, **kwargs):
        pass


def check_dbus_interface(service_name, path, interface_name):
    original_stderr = sys.stderr
    sys.stderr = NullWriter()
    try:
        bus = dbus.SystemBus()
        # 尝试获取具有特定接口的对象
        obj = bus.get_object(service_name, path)
        dbus_iface = dbus.Interface(obj, "org.freedesktop.DBus.Introspectable")
        # 尝试调用接口的方法
        dbus_iface.Introspect()
        sys.stderr = original_stderr
        return True
    except Exception as error_message:
        # 如果出现任何 DBus 相关的错误，认为接口不存在
        sys.stderr = original_stderr
        return False
    # except dbus.exceptions.DBusException as error_message:
    #     return False


# ---------------------------------------------------------------------

dbusServerList = [
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/appmanager",
        "interface": "com.ydjw.mdm.appmanager",
    },
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/systemmanager",
        "interface": "com.ydjw.mdm.systemmanager",
    },
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/networkmanager",
        "interface": "com.ydjw.mdm.networkmanager",
    },
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/storage",
        "interface": "com.ydjw.mdm.storage",
    },
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/systeminfo",
        "interface": "com.ydjw.mdm.systeminfo",
    },
    {
        "service": "com.ydjw.mdm",
        "path": "/com/ydjw/mdm/device",
        "interface": "com.ydjw.mdm.device",
    },
]

print("checking dbus interface availability ...")

is_dbus_available = True

for interface in dbusServerList:
    sleep_time = 0
    while True:
        if not check_dbus_interface(
            interface["service"], interface["path"], interface["interface"]
        ):
            print(
                "\rWaiting for dbus interface (%s) to be available ... %ds"
                % (interface["path"], sleep_time),
                end="",
            )
            time.sleep(1)
            sleep_time += 1
            if sleep_time > 300:
                print("dbus interface (%s) is not available ..." % interface["path"])
                is_dbus_available = False
                break
        else:
            print("\ndbus interface (%s) is available ..." % interface["path"], end="")
            break

if not is_dbus_available:
    print("\rdbus interface is not available ...")
    exit(1)


print("\ndbus interface initialized ...")
# ---------------------------------------------------------------------

system_bus = dbus.SystemBus()
# ---------------------------------------------------------------------
# 获取 当前接口是否需要进行授权
try:
    system = system_bus.get_object('com.ydjw.mdm',
                                   '/com/ydjw/mdm/systemmanager')
    system_interface = dbus.Interface(
        system, dbus_interface='com.ydjw.mdm.systemmanager')
    interface_authority_switch=system_interface.GetKydbusInterfaceAuthoritySwitch()
    print(
        "\ngets whether the interface needs to be authorized ... (%s)"
        % "\033[031mAuthorization required\033[0m"
        if interface_authority_switch
        else "\033[031mNo authorization is required\033[0m"
    )
except Exception as error:
    pass

# ---------------------------------------------------------------------
# 设置 软件安装黑白名单安装 默认模式为 黑名单
try:
    appmanager = system_bus.get_object('com.ydjw.mdm',
                                       '/com/ydjw/mdm/appmanager')
    appmanager_interface = dbus.Interface(
        appmanager, dbus_interface='com.ydjw.mdm.appmanager')

    print("set the software installation control mode ... %s" % (
        Succeed_String if (appmanager_interface.SetSoftwareInstBwMode(1,kydbus_police_key) if interface_authority_switch == True else appmanager_interface.SetSoftwareInstBwMode(1)) == 0 else Failed_String))  # 设置软件安装黑白名单安装模式 # 0为不管控 2为白名单 1为黑名单
except Exception as error:
    pass


try:
    appmanager = system_bus.get_object('com.ydjw.mdm',
                                       '/com/ydjw/mdm/appmanager')
    appmanager_interface = dbus.Interface(
        appmanager, dbus_interface='com.ydjw.mdm.appmanager')

    print("set the software installation control mode ... %s" % (
        Succeed_String if (appmanager_interface.SetSoftwareInstallBwMode(1,kydbus_police_key) if interface_authority_switch == True else appmanager_interface.SetSoftwareInstallBwMode(1)) == 0 else Failed_String))  # 设置软件安装黑白名单安装模式 # 0为不管控 2为白名单 1为黑名单
except Exception as error:
    pass

# ---------------------------------------------------------------------
# 设置 无线共享状态
try:
    appmanager = system_bus.get_object('com.ydjw.mdm',
                                       '/com/ydjw/mdm/networkmanager')
    appmanager_interface = dbus.Interface(
        appmanager, dbus_interface='com.ydjw.mdm.networkmanager')

    print("set the net connect state ... %s" % (
        Succeed_String if (appmanager_interface.SetNetConnectState(1,kydbus_police_key) if interface_authority_switch == True else appmanager_interface.SetNetConnectState(1)) == 0 else Failed_String))  # 设置软件安装黑白名单安装模式 # 0为不管控 2为白名单 1为黑名单
except Exception as error:
    pass

# ---------------------------------------------------------------------

try:
    device = system_bus.get_object('com.ydjw.mdm',
                                   '/com/ydjw/mdm/device')
    device_interface = dbus.Interface(
        device, dbus_interface='com.ydjw.mdm.device')
    
    # 设置 红外设备 默认控制状态为 禁用
    print("set the infrared device control status ... %s" % (
        Succeed_String if (device_interface.ControlInfraredDevice(0,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlInfraredDevice(0)) ==0 else Failed_String))

    # 设置 串口设备 默认控制状态为 启用
    print("set the serial device control status ... %s" % (
        Succeed_String if (device_interface.ControlSerialDevice(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlSerialDevice(1)) == 0 else Failed_String))
    
    print("initializing the bluetooth connection whitelist ... %s" % (
        Succeed_String if (device_interface.SetBluetoothWhiteList([],kydbus_police_key) if interface_authority_switch == True else device_interface.SetBluetoothWhiteList([])) == 0 else Failed_String))

    # 设置 蓝牙设备 默认控制状态为 启用
    print("set the bluetooth device control status ... %s" % (
        Succeed_String if (device_interface.ControlBluetoothDevice(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlBluetoothDevice(1)) == 0 else Failed_String))
    
    # 初始化 蓝牙连接白名单
    print("initializing the bluetooth connection whitelist ... %s" % (
        Succeed_String if (device_interface.SetBluetoothWhiteList([],kydbus_police_key) if interface_authority_switch == True else device_interface.SetBluetoothWhiteList([])) == 0 else Failed_String))

    # 设置 MinCard 默认控制状态为 启用
    print("set the MinCard device control status ... %s" % (
        Succeed_String if (device_interface.ControlMinCardInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlMinCardInterface(1)) == 0 else Failed_String))
    
    # 设置 有线 默认控制状态为 启用
    print("set the Wired device control status ... %s" % (
        Succeed_String if (device_interface.ControlWiredInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlWiredInterface(1)) == 0 else Failed_String))
    
    # 设置 无线 默认控制状态为 启用
    print("set the Wireless device control status ... %s" % (
        Succeed_String if (device_interface.ControlWirelessInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlWirelessInterface(1)) == 0 else Failed_String))
    
    # 设置 显示设备 默认控制状态为 启用
    print("set the Display device control status ... %s" % (
        Succeed_String if (device_interface.ControlDisplayInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlDisplayInterface(1)) == 0 else Failed_String))
    
    # 设置 Mic 默认控制状态为 启用
    print("set the Mic device control status ... %s" % (
        Succeed_String if (device_interface.ControlMicInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlMicInterface(1)) == 0 else Failed_String))
    
    
    # 设置 定位设备 默认控制状态为 启用
    # print("set the Location device control status ... %s" % (
    #     Succeed_String if (device_interface.ControlLocationInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlLocationInterface(1)) == 0 else Failed_String))
    
    

    # 设置 相机 默认控制状态为 启用
    print("set the Camera device control status ... %s" % (
        Succeed_String if (device_interface.ControlCameraInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlCameraInterface(1)) == 0 else Failed_String))
    

    # 设置 指纹设备 默认控制状态为 启用
    print("set the Fingerprint device control status ... %s" % (
        Succeed_String if (device_interface.ControlFingerprintInterface(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlFingerprintInterface(1)) == 0 else Failed_String))
    
    

    # 设置 红外设备 默认控制状态为 启用
    print("set the Fingerprint device control status ... %s" % (
        Succeed_String if (device_interface.ControlInfraredDevice(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlInfraredDevice(1)) == 0 else Failed_String))
    
    

    # 设置 光驱设备 默认控制状态为 启用
    print("set the CD device control status ... %s" % (
        Succeed_String if (device_interface.ControlCDDevice(1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlCDDevice(1)) == 0 else Failed_String))

    device_types = [
        1,  # KYSEC_USB - usb接口
        2,  # KYSEC_SERIAL - 串口
        3,  # KYSEC_PARALLEL - 并口
        4,  # KYSEC_TYPEC - type-c接口
        5,  # KYSEC_CDROM - 光驱（内置光驱）
        6,  # KYSEC_PRINTER - 打印机类
        7,  # KYSEC_WIRELESS - 无线网络
        8,  # KYSEC_ETHERNET - 有线网络
        9,  # KYSEC_BLUETOOTH - 蓝牙
        10,  # KYSEC_CAMERA - 摄像头
        11,  # KYSEC_SCANNER - 扫描仪
        12,  # KYSEC_AUDIO - 音频(目前仅管控到3.5mm jack)
        13,  # KYSEC_IEEE1394 - IEEE1394接口
        14,  # KYSEC_PCMCIA - PCMCIA接口
        15,  # KYSEC_PCIE - PCIE接口
        16,  # KYSEC_SATA - SATA接口
        17,  # KYSEC_PS2 - PS2接口 （暂未提供）
        18,  # KYSEC_VGA - VGA接口 （暂未提供）
        19,  # KYSEC_HDMI - HDMI接口
        20,  # KYSEC_DP - DP接口
        21,  # KYSEC_INFRARED - 红外设备
        22,  # KYSEC_MODEM - 网络modem
        23,  # KYSEC_MMC - mmc设备
        24,  # KYSEC_SCSI_RAID - scsi_raid设备
        25,  # KYSEC_FLOPPY_DRIVE - 软驱设备
        26,  # KYSEC_VIDEO - 图像设备
        27,  # KYSEC_DIGITAL_CAMERA - 数码相机设备
        28,  # KYSEC_MULTI_SERIAL_CARD - 多串口
        29,  # KYSEC_WINCE - wince设备(手机、ipad等)
        30,  # KYSEC_PRINTER_LOCAL - 本地打印机
        31,  # KYSEC_PRINTER_NET - 网络打印机
        32,  # KYSEC_FILE_SHARING - 暂未使用（sm）
        33,  # KYSEC_BATTERY_AND_UPS - 暂未使用（sm）
        34,  # KYSEC_BIOMETRIC - 暂未使用（sm）
        35,  # KYSEC_DISK_DRIVE - 暂未使用（sm）
        36,  # KYSEC_DISPLAY_CARD - 暂未使用（sm）
        37,  # KYSEC_HDC - 暂未使用（sm）
        38,  # KYSEC_KEYBOARD - 暂未使用（sm）
        39,  # KYSEC_MOUSE - 暂未使用（sm）
        40,  # KYSEC_STORAGE_VOLUME - 暂未使用（sm）
        41,  # KYSEC_CDDRIVE - 暂未使用（sm）
        42,  # KYSEC_VOLUME_SHADOW - 卷影设备 暂未使用（sm）
        43,  # KYSEC_TAPE - 磁带机 暂未使用（sm）
    ]
    # 设置 USB所有类型都为开启状态
    print("set the all usb type device control status ... %s" % (
        Succeed_String if (device_interface.ControlUsbDeviceType(device_types,1,kydbus_police_key) if interface_authority_switch == True else device_interface.ControlUsbDeviceType(device_types,1)) == 0 else Failed_String))

except Exception as error:
    pass
# ---------------------------------------------------------------------

# ---------------------------------------------------------------------
# 将识别出的网卡添加到网卡白名单中,该脚本仅在安装时执行,因此,仅在安装时,会将主机上识别到的网卡加入到网卡白名单中

def get_mac_addresses():
    interfaces = psutil.net_if_addrs()

    mac_addresses = {}
    print("Interface List:")
    for interface, addresses in interfaces.items():
        for address in addresses:
            if address.family == psutil.AF_LINK and address.address != "00:00:00:00:00:00" and address.address != "00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00":
                mac_addresses[interface] = address.address
                print("\t%s  %s"%(interface,address.address))
    return mac_addresses

mac_list = []
try:
    mac_addresses = get_mac_addresses()
    for interface, mac_address in mac_addresses.items():
        mac_list.append(mac_address)
except Exception as error_message:
    pass
    print(error_message)

try:
    networkmanager = system_bus.get_object(
        'com.ydjw.mdm', '/com/ydjw/mdm/networkmanager')
    networkmanager_interface = dbus.Interface(
        networkmanager, dbus_interface='com.ydjw.mdm.networkmanager')
    print("add the identified network adapter to the network adapter whitelist ... %s" % (
        Succeed_String if (networkmanager_interface.SetNetDeviceList(mac_list,kydbus_police_key) if interface_authority_switch == True else networkmanager_interface.SetNetDeviceList(mac_list)) == 0 else Failed_String))
except Exception as error:
    pass


# ---------------------------------------------------------------------
# 设置 网络管理器 无线连接自动重连

# 定义配置文件路径和备份文件路径
file_path = '/etc/NetworkManager/NetworkManager.conf'
backup_path = '/etc/NetworkManager/NetworkManager.conf.kydbus-police.backend'

print("check the content of the configuration file %s ... " % file_path,end='')

# 创建配置文件的备份
if not os.path.exists(backup_path):
    shutil.copyfile(file_path, backup_path)

# 读取并修改配置文件
connection_section_exists = False
connection_section_context_exists = False
connection_lines = []

with open(file_path, 'r') as file:
    lines = file.readlines()
    for line in lines:
        if '[connection]' in line:
            connection_section_exists = True
        if 'wifi.autoconnect' in line:
            connection_section_context_exists = True
        connection_lines.append(line)


if connection_section_context_exists:
    for i, line in enumerate(connection_lines):
        if 'wifi.autoconnect' in line:
            if 'true' not in line:
                connection_lines[i] = 'wifi.autoconnect=true\n'
            break
else:
    if not connection_section_exists:
        connection_lines.append('\n[connection]\n')
        connection_lines.append('wifi.autoconnect=true\n')
    else:
        for i, line in enumerate(connection_lines):
            if '[connection]' in line:
                connection_lines.insert(i+1, "wifi.autoconnect=true\n")
                break

# 将修改后的内容写回文件
with open(file_path, 'w') as file:
    file.writelines(connection_lines)

# ---------------------------------------------------------------------

# 执行重启 NetworkManager 的命令
try:
    subprocess.run(['systemctl', 'restart', 'NetworkManager'], check=True)
    print("NetworkManager 服务已重启")
except subprocess.CalledProcessError as e:
    print(f"重启 NetworkManager 服务失败：{e}")

# ---------------------------------------------------------------------


os.remove(kydbus_police_json)
kydbus_installed = "/etc/kydbus-police/.Installed.tmp"
if os.path.exists(kydbus_installed):
    os.remove(kydbus_installed)
